import React, { useState, useEffect } from "react";
import { useScrollDirection } from "@hooks/useScrollDirection";
import { IHeader } from "@interfaces/IHeader";
import { LogoResponsive } from "@components/LogoResponsive";
import { MobileMenu } from "@components/Header/components/MobileMenu";
import { HamburgerButton } from "./components/HamburgerButton";
import { TopBar } from "@components/TopBar";
import { Toolbar } from "@components/Toolbar";
import { LanguageSelector } from "@components/LanguageSelector";
import { LinkOrContainer } from "@components/LinkOrContainer";
import { Logo } from "@components/Logo";
import { clsx } from "clsx";

export type HeaderProps = IHeader & {
  backgroundImageClasses?: string;
  customLogo?: [string, string];
  customLogoAlt?: [string, string];
  logoAlwaysInTopBar?: boolean;
  organization?: "fchh" | "frbs";
};

export function Header({
  className,
  topBarItems,
  toolBarItems,
  extendedToolBarItems,
  onClickHamburger,
  languages,
  onChangeLanguage,
  selectedLanguage,
  onSearch,
  disableAnimation,
  backgroundImageClasses,
  itemsAlwaysLight,
  hideSearchIcon,
  logoLinkTarget = "/",
  logoAlwaysInTopBar,
  customLogo,
  customLogoAlt,
  logoClassName = "",
  organization = "fchh",
}: HeaderProps) {
  const [shouldShowHeader, setShouldShowHeader] = useState(true);
  const [isMobileOpen, setIsMobileOpen] = useState(false);
  const scrollDirection = useScrollDirection();

  useEffect(() => {
    if (!isMobileOpen && !disableAnimation) {
      setShouldShowHeader(!scrollDirection);
    }
  }, [scrollDirection]);

  const handleMobileMenuClick = () => {
    setIsMobileOpen(!isMobileOpen);
    onClickHamburger && onClickHamburger();
  };

  const alwaysLight = itemsAlwaysLight || isMobileOpen;

  const CustomLogo = ({
    dark = true,
    className = "",
  }: {
    dark?: boolean;
    className?: string;
  }) =>
    customLogo?.[0] ? (
      <img
        className={clsx("fs-h-full fs-object-contain", className)}
        src={dark ? customLogo?.[0] : customLogo?.[1]}
        alt={dark ? customLogoAlt?.[0] : customLogoAlt?.[0]}
      />
    ) : null;

  const getTopBarLogo = (
    logoAlwaysInTopBar?: boolean,
    customLogo?: React.ReactNode,
  ): React.ReactNode => {
    if (logoAlwaysInTopBar && customLogo) {
      return (
        <LinkOrContainer href={logoLinkTarget}>
          <CustomLogo className={logoClassName} />
        </LinkOrContainer>
      );
    } else if (logoAlwaysInTopBar && !customLogo) {
      return (
        <LinkOrContainer href={logoLinkTarget}>
          <Logo
            lightColor={false}
            monotone={false}
            className={"!fs-h-6 fs-block"}
            organization={organization}
          />
        </LinkOrContainer>
      );
    } else {
      return null;
    }
  };

  return (
    <div
      className={clsx(
        "fs-w-full fs-fixed sm:fs-h-auto fs-z-50 fs-transition-all fs-ease-in-out fs-left-0 fs-duration-500",
        isMobileOpen && "!fs-h-screen",
        backgroundImageClasses
          ? isMobileOpen
            ? "fs-bg-grey-900"
            : backgroundImageClasses
          : isMobileOpen
            ? "fs-bg-grey-900"
            : "fs-bg-white lg:fs-bg-transparent",
        shouldShowHeader ? "fs-top-0" : "-fs-top-64",
        className,
      )}
    >
      <TopBar
        items={topBarItems}
        onChangeLanguage={onChangeLanguage}
        selectedLanguage={selectedLanguage}
        languages={languages}
        onSearch={(keyword) => onSearch && onSearch(keyword)}
        light={itemsAlwaysLight}
        hideSearchIcon={hideSearchIcon}
        customLogo={getTopBarLogo(logoAlwaysInTopBar, customLogo)}
      />
      <div className="container fs-flex fs-justify-between fs-items-center fs-py-3 lg:fs-py-6">
        {customLogo && (
          <LinkOrContainer
            className={`fs-w-auto lg:fs-w-36 lg:fs-h-auto fs-h-10 fs-z-10 fs-overflow-hidden lg:fs-overflow-auto ${
              logoAlwaysInTopBar ? "lg:fs-hidden" : ""
            }`}
            href={logoLinkTarget}
          >
            <CustomLogo dark={!isMobileOpen} className={logoClassName} />
          </LinkOrContainer>
        )}
        {!customLogo && (
          <LogoResponsive
            desktopLight={true}
            mobileLight={alwaysLight}
            desktopMonotone={true}
            mobileMonotone={false}
            desktopClasses={logoAlwaysInTopBar ? "!fs-hidden" : ""}
            href={logoLinkTarget}
            organization={organization}
          />
        )}
        <div className="flex items-center gap-7">
          {languages?.length && (
            <LanguageSelector
              isMobileOpen={isMobileOpen}
              onChangeLanguage={onChangeLanguage}
              selectedLanguage={selectedLanguage}
              languages={languages}
              light={alwaysLight}
              horizontal
            />
          )}
          <HamburgerButton
            onClick={handleMobileMenuClick}
            light={alwaysLight}
          />
        </div>
        {toolBarItems && (
          <Toolbar isMobileOpen={isMobileOpen} items={toolBarItems} />
        )}
      </div>
      {/* Toolbar in mobile menu */}
      {toolBarItems && (
        <Toolbar
          isMobileOpen={isMobileOpen}
          items={toolBarItems}
          extendedToolbarItems={extendedToolBarItems}
          className="lg:fs-hidden"
        />
      )}

      <MobileMenu isMobileOpen={isMobileOpen} data={topBarItems} />
    </div>
  );
}
