import { CSSProperties } from "react";
import { Pill } from "@components/Pill";
import { Time, Location } from "@carbon/icons-react";
import { INewsCard } from "@interfaces/INews";
import { Link } from "@components/Link";
import clsx from "clsx";
import { CommonUtils } from "@utils/CommonUtils";

type CardColorVariant = "dark" | "light";
type PillColorType = "dark" | "white";

export function HorizontalNewsCard({
  image,
  title,
  description,
  date,
  location,
  category,
  href,
  variant,
  className = "",
  customBackgroundColor,
  textColor = "dark",
  newTab,
  id,
}: INewsCard) {
  const inlineStyle: CSSProperties = {
    backgroundColor: customBackgroundColor,
  };
  const PILL_COLOR_MAPPING: Record<CardColorVariant, PillColorType> = {
    dark: "white",
    light: "dark",
  };

  const STYLE_MAPPING: Record<string, Record<string, string>> = {
    base: {
      outerContainer: "fs-flex fs-h-72",
      contentContainer:
        "fs-group fs-order-1 fs-flex-1 fs-flex fs-flex-col fs-justify-between fs-p-7",
      linkContainer: "fs-mb-5 sm:fs-mb-0",
    },
    dark: {
      outerContainer: "fs-bg-grey-900",
      contentContainer: "",
      linkContainer: "",
    },
    light: {
      outerContainer: "fs-bg-pearl-500",
      contentContainer: "",
      linkContainer: "",
    },
    custom: {
      outerContainer: "",
      contentContainer: "",
      linkContainer: "",
    },
  };
  const TEXT_STYLE_MAPPING: Record<string, Record<string, string>> = {
    base: {
      heading:
        "group-peer-hover:fs-multi-line-underline group-hover:fs-multi-line-underline fs-font-karla fs-text-2xl md:fs-text-3xl fs-leading-8 md:fs-leading-9 fs-font-bold fs-tracking-tight fs-tracking-tight fs-line-clamp-2 sm:fs-line-clamp-3 xl:fs-line-clamp-none",
      paragraph:
        "fs-mt-2 md:fs-mt-3 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal fs-line-clamp-3 lg:fs-line-clamp-4 xl:fs-line-clamp-3",
      event:
        "fs-mt-2 md:fs-mt-3 fs-font-plex fs-text-base md:fs-text-lg fs-leading-6 md:fs-leading-7 fs-font-normal",
    },
    dark: {
      heading: "xl:fs-multi-line-3-white fs-text-white",
      paragraph: "fs-text-grey-100",
      event: "fs-text-grey-50",
    },
    light: {
      heading: "xl:fs-multi-line-3-grey-900 fs-text-grey-900",
      paragraph: "fs-text-grey-900",
      event: "fs-text-grey-900",
    },
  };

  return (
    <div
      style={inlineStyle}
      className={clsx(
        variant
          ? STYLE_MAPPING[variant]["outerContainer"]
          : STYLE_MAPPING["custom"]["outerContainer"],
        STYLE_MAPPING["base"]["outerContainer"],
        className,
      )}
      id={id}
    >
      <div className="fs-peer fs-order-2 fs-hidden md:fs-block fs-flex-initial fs-h-full fs-w-auto">
        <Link
          to={href}
          {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
          className="fs-h-full fs-w-auto"
        >
          <img
            className="fs-h-full fs-w-auto fs-object-cover fs-aspect-square fs-object-center"
            alt={image.alt}
            src={image.src}
          />
        </Link>
      </div>
      <div
        className={clsx(
          variant
            ? STYLE_MAPPING[variant]["contentContainer"]
            : STYLE_MAPPING["custom"]["contentContainer"],
          STYLE_MAPPING["base"]["contentContainer"],
        )}
      >
        <Link
          to={href}
          {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
          className={clsx(
            variant
              ? STYLE_MAPPING[variant]["linkContainer"]
              : STYLE_MAPPING["custom"]["linkContainer"],
            STYLE_MAPPING["base"]["linkContainer"],
          )}
        >
          <h5
            className={clsx(
              variant
                ? TEXT_STYLE_MAPPING[variant]["heading"]
                : TEXT_STYLE_MAPPING[CommonUtils.invertDarkLight(textColor)][
                    "heading"
                  ],
              TEXT_STYLE_MAPPING["base"]["heading"],
            )}
          >
            {title}
          </h5>

          {description && (
            <p
              className={clsx(
                variant
                  ? TEXT_STYLE_MAPPING[variant]["paragraph"]
                  : TEXT_STYLE_MAPPING[CommonUtils.invertDarkLight(textColor)][
                      "paragraph"
                    ],
                TEXT_STYLE_MAPPING["base"]["paragraph"],
              )}
            >
              {description}
            </p>
          )}
          {!description && (date || location) && (
            <div
              className={clsx(
                variant
                  ? TEXT_STYLE_MAPPING[variant]["event"]
                  : TEXT_STYLE_MAPPING[CommonUtils.invertDarkLight(textColor)][
                      "event"
                    ],

                TEXT_STYLE_MAPPING["base"]["event"],
              )}
            >
              {date && (
                <span className="fs-flex fs-items-center">
                  <Time size={16} className="fs-mr-1 md:fs-mr-2" />
                  {date}
                </span>
              )}
              {location && (
                <span className="fs-flex fs-items-center fs-mt-1">
                  <Location size={16} className="fs-mr-1 md:fs-mr-2" />
                  {location}
                </span>
              )}
            </div>
          )}
        </Link>
        <div className="fs-hidden md:fs-flex fs-flex-wrap">
          {category?.title && (
            <Pill
              type={
                variant
                  ? PILL_COLOR_MAPPING[variant]
                  : PILL_COLOR_MAPPING[CommonUtils.invertDarkLight(textColor)]
              }
              size="sm"
              rounded
              href={category.href}
              title={category.title}
            />
          )}
        </div>
      </div>
    </div>
  );
}
