import { useState } from "react";
import { Link } from "@components/Link";
import { ChevronDown, ChevronRight } from "@carbon/icons-react";
import { ITopBar, ITopBarItem } from "@interfaces/IHeader";
import { LanguageSelector } from "@components/LanguageSelector";
import { Search } from "@components/Search";
import clsx from "clsx";
import { ConditionalWrapper } from "./ConditionalWrapper";

export function TopBar({
  items,
  languages,
  onChangeLanguage,
  selectedLanguage,
  onSearch,
  light,
  hideSearchIcon,
  customLogo,
  logoClassName,
}: ITopBar) {
  const [shouldFadeOthers, setShouldFadeOthers] = useState(false);
  return items.length ? (
    <nav className="fs-w-full fs-relative fs-hidden lg:fs-inline-block fs-bg-white fs-border-b fs-border-black">
      <div
        className={clsx(
          "container fs-flex fs-items-center",
          customLogo ? "fs-justify-between" : "fs-justify-end",
        )}
      >
        {Boolean(customLogo) && (
          <div
            className={clsx(
              "fs-w-auto fs-hidden lg:fs-block lg:fs-w-36 lg:fs-h-auto fs-h-8 fs-z-10 fs-overflow-hidden lg:fs-overflow-auto",
              logoClassName,
            )}
          >
            {customLogo}
          </div>
        )}
        <ul className="fs-flex fs-float-right fs-gap-x-6 fs-w-auto">
          {items.map((item, index) => (
            <TopBarItem
              key={index}
              {...item}
              faded={shouldFadeOthers}
              onHover={(value) => setShouldFadeOthers(value)}
              levelThreeToTheLeft={index >= Math.round((items.length - 1) / 2)}
              firstItem={index === 0}
            />
          ))}
          {!hideSearchIcon && (
            <li className="fs-flex fs-items-center xl:fs-ml-20">
              <Search
                onSearch={(keyword) => onSearch && onSearch(keyword)}
                faded={shouldFadeOthers}
                onHover={(value) => setShouldFadeOthers(value)}
              />
            </li>
          )}
          {languages && languages?.length > 1 ? (
            <li className="fs-flex fs-items-center fs-ml-5">
              <LanguageSelector
                languages={languages}
                selectedLanguage={selectedLanguage}
                onChangeLanguage={onChangeLanguage}
                faded={shouldFadeOthers}
                onHover={(value) => setShouldFadeOthers(value)}
                light={light}
              />
            </li>
          ) : null}
        </ul>
      </div>
    </nav>
  ) : null;
}

function TopBarItem({
  title,
  target,
  children,
  levelThreeToTheLeft,
  faded,
  onHover,
  firstItem,
}: ITopBarItem) {
  const classes =
    "fs-group fs-py-2 fs-relative fs-text-sm lg:fs-text-base fs-leading-6 fs-font-medium fs-flex-1 fs-text-center xl:fs-text-right fs-whitespace-nowrap xl:fs-ml-20 fs-transition-all fs-ease-in fs-duration-100 hover:fs-text-grey-900";
  return (
    <li
      onMouseEnter={() => onHover && onHover(true)}
      onMouseLeave={() => onHover && onHover(false)}
      className={clsx(
        classes,
        faded ? "fs-text-grey-400" : "fs-text-grey-900",
        target && "fs-cursor-pointer",
      )}
    >
      <ConditionalWrapper
        condition={Boolean(target)}
        wrapper={(children) => <Link to={target}>{children}</Link>}
      >
        <span className="fs-flex fs-items-center fs-justify-center xl:fs-justify-end">
          {title}
          {children && children.length > 0 && (
            <ChevronDown
              size={16}
              className="fs-ml-2 group-hover:fs-opacity-15 fs-transition-opacity fs-ease-in-out fs-duration-100"
            />
          )}
        </span>
      </ConditionalWrapper>
      {children && children.length > 0 && (
        <div
          className={clsx(
            "fs-z-20 group-hover:fs-block fs-hidden fs-absolute fs-top-10 xl:-fs-left-7 fs-w-[273px] fs-bg-grey-900",
            firstItem ? "-fs-left-4" : "fs-left-5",
          )}
        >
          <div className="fs-flex fs-flex-col fs-items-start fs-font-plex fs-text-base fs-leading-6 fs-font-medium fs-text-grey-600">
            {children.map((levelTwoChild, index) => {
              const linkClasses = clsx(
                "fs-peer fs-relative fs-block fs-w-full fs-px-7 fs-whitespace-normal",
                index === 0
                  ? "fs-pt-7 fs-pb-3"
                  : index === children.length - 1
                    ? "fs-pt-3 fs-pb-7"
                    : "fs-py-3",
                children.length === 1 && "fs-pb-7",
              );
              return (
                <div
                  key={index}
                  className="fs-w-full fs-relative fs-text-left hover:fs-text-white"
                >
                  {levelTwoChild.target ? (
                    <Link className={linkClasses} to={levelTwoChild.target}>
                      {levelTwoChild.title}
                      {levelTwoChild.children &&
                        levelTwoChild.children.length > 0 && (
                          <ChevronRight
                            size={20}
                            className={`fs-absolute fs-right-3 -fs-translate-y-2/4 ${
                              index === 0
                                ? "fs-top-[65%]"
                                : index === children.length - 1
                                  ? "fs-top-[40%]"
                                  : "fs-top-1/2"
                            }`}
                          />
                        )}
                    </Link>
                  ) : (
                    <span className={linkClasses}>
                      {levelTwoChild.title}
                      {levelTwoChild.children &&
                        levelTwoChild.children.length > 0 && (
                          <ChevronRight
                            size={20}
                            className={`fs-absolute fs-right-3 -fs-translate-y-2/4 ${
                              index === 0
                                ? "fs-top-[65%]"
                                : index === children.length - 1
                                  ? "fs-top-[40%]"
                                  : "fs-top-1/2"
                            }`}
                          />
                        )}
                    </span>
                  )}

                  {levelTwoChild.children &&
                    levelTwoChild.children.length > 0 && (
                      <div
                        className={clsx(
                          "peer-hover:fs-block hover:fs-block fs-hidden fs-absolute fs-z-10 fs-w-[274px]",
                          index > 0 ? "-fs-top-4" : "fs-top-0",
                          levelThreeToTheLeft
                            ? "fs-right-full"
                            : "fs-left-full",
                        )}
                      >
                        <div className="fs-flex fs-flex-col fs-items-start fs-bg-grey-900 fs-font-plex fs-text-base fs-leading-6 fs-font-medium fs-text-grey-600">
                          {levelTwoChild.children?.map(
                            (levelThreeChild, index) => (
                              <div
                                key={index}
                                className="fs-w-full fs-text-left hover:fs-text-white"
                              >
                                <Link
                                  className={clsx(
                                    "fs-block fs-w-full fs-relative fs-px-7 fs-whitespace-normal",
                                    index === 0
                                      ? "fs-pt-7 fs-pb-3"
                                      : index ===
                                          (levelTwoChild.children &&
                                            levelTwoChild.children.length - 1)
                                        ? "fs-pt-3 fs-pb-7"
                                        : "fs-py-3",
                                    levelTwoChild.children?.length === 1 &&
                                      "fs-pb-7",
                                  )}
                                  to={levelThreeChild.target}
                                >
                                  {levelThreeChild.title}
                                </Link>
                              </div>
                            ),
                          )}
                        </div>
                      </div>
                    )}
                </div>
              );
            })}
          </div>
        </div>
      )}
    </li>
  );
}
